
#
# v0.2
# ericc@a10networks.com
#

import json, urllib2, ssl

def axapi_auth(host, username, password):
    base_uri = 'https://'+host
    auth_payload = {"credentials": {"username": username, "password": password}}
    r = axapi_action(base_uri + '/axapi/v3/auth', payload=auth_payload)
    signature = json.loads(r)['authresponse']['signature']
    return base_uri, signature


def axapi_action(uri, payload='', signature='', method='POST'):
    # PEP476  2.7.9+ / 3.4.3+ cert check
    new_context = ssl._create_unverified_context()
    try:
        if method == 'POST':
            req = urllib2.Request(uri)
            req.add_header('content-type', 'application/json')
            if signature:
                req.add_header('Authorization', 'A10 {0}'.format(signature))
            response = urllib2.urlopen(req, json.dumps(payload), context=new_context)
        elif method == 'GET':
            req = urllib2.Request(uri)
            req.add_header('content-type', 'application/json')
            if signature:
                req.add_header('Authorization', 'A10 {0}'.format(signature))
            response = urllib2.urlopen(req, context=new_context)
        elif method == 'DELETE':
            req = urllib2.Request(uri)
            req.add_header('content-type', 'application/json')
            req.get_method = lambda: 'DELETE'
            if signature:
                req.add_header('Authorization', 'A10 {0}'.format(signature))
            response = urllib2.urlopen(req, context=new_context)
        return response.read()
    except Exception as e:
        raise 






#!/usr/bin/python

#
# Eric Chou (ericc@a10networks.com)
#

import sys
from sys import stdin
import optparse
import logging, json
from a10 import axapi_auth, axapi_action
from json_configs.logoff import logoff_path
from json_configs.write_memory import write_mem_path
from json_configs.ddos_dst_zone import ddos_dst_zone_path, ddos_dst_zone

LOG_FILE = "/var/log/fastnetmon-notify.log"


logger = logging.getLogger("DaemonLog")
logger.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
handler = logging.FileHandler(LOG_FILE)
handler.setFormatter(formatter)
logger.addHandler(handler)



client_ip_as_string=sys.argv[1]
data_direction=sys.argv[2]
pps_as_string=int(sys.argv[3])
action=sys.argv[4]

logger.info(" - " . join(sys.argv))


# A10 Mitigator Information
mitigator_ip = "192.168.199.150"
zone_name = client_ip_as_string + "_zone"
ip_addr = client_ip_as_string
mitigator_base_url, signature = axapi_auth(mitigator_ip, "admin", "a10")


if action == "unban":
    try: 
        r = axapi_action(mitigator_base_url+ddos_dst_zone_path+zone_name, method="DELETE", signature=signature)
    except Exception as e: 
        logger.info("Zone not removed in unban, may not exist. Result: " + str(e))

    # Commit config
    axapi_action(mitigator_base_url+write_mem_path, signature=signature)
    # Logoff
    axapi_action(mitigator_base_url+logoff_path, signature=signature)
 
    sys.exit(0)

elif action == "ban":
    
    r = axapi_action(mitigator_base_url+ddos_dst_zone_path, method='GET', signature=signature)
    try: 
        if zone_name in [i['zone-name'] for i in json.loads(r)['zone-list']]:
            r = axapi_action(mitigator_base_url+ddos_dst_zone_path+zone_name, method="DELETE", signature=signature)
            logger.info(str(r))
    except Exception as e:
        logger.info("No Zone detected or something went wrong. Erorr: " + str(e))

    # A10 Mitigation On Ramp 
    zone_name = client_ip_as_string + "_zone"
    ip_addr = client_ip_as_string
    returned_body = ddos_dst_zone(zone_name, ip_addr)
    try:
        r = axapi_action(mitigator_base_url+ddos_dst_zone_path, signature=signature, payload=returned_body)
    except Exception as e:
        logger.info("zone not created: " + str(e))

    # Commit changes
    axapi_action(mitigator_base_url+write_mem_path, signature=signature)
    # Log off
    axapi_action(mitigator_base_url+logoff_path, signature=signature)
    
    sys.exit(0)

else:
    sys.exit(0)








ddos_dst_zone_path = '/axapi/v3/ddos/dst/zone/'

def ddos_dst_zone(zone_name, ip_addr):
    ddos_dst_zone_payload = {
    "zone-list": [
    {
      "zone-name":zone_name,
      "ip": [
        {
          "ip-addr": ip_addr,
        }
      ],
      "operational-mode":"monitor",
      "advertised-enable":1,
      "zone-template": {
        "logging":"cef-logger"
      },
      "log-enable":1,
      "log-periodic":1,
      "ip-proto": {
        "proto-tcp-udp-list": [
          {
            "protocol":"tcp",
            "drop-frag-pkt":1,
          },
          {
            "protocol":"udp",
            "drop-frag-pkt":1,
          }
        ],
        "proto-name-list": [
          {
            "protocol":"icmp-v4",
            "deny":1,
            "detection-enable":1,
          },
          {
            "protocol":"icmp-v6",
            "deny":1,
            "detection-enable":1,
          }
        ]
      },
      "port": {
        "zone-service-list": [
          {
            "port-num":20,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":21,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":22,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":25,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":53,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":53,
            "protocol":"udp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "udp":"udp-protect1"
                },
              }
            ]
          },
          {
            "port-num":80,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":110,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":143,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":443,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":587,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":993,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":995,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":5060,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          },
          {
            "port-num":5061,
            "protocol":"tcp",
            "detection-enable":1,
            "level-list": [
              {
                "level-num":"0",
                "zone-escalation-score":10,
                "indicator-list": [
                  {
                    "type":"pkt-rate",
                    "score":20,
                    "zone-threshold-num":1,
                  }
                ]
              },
              {
                "level-num":"1",
                "zone-template": {
                  "tcp":"tcp-protect1"
                },
              }
            ]
          }
        ],
        "zone-service-other-list": [
          {
            "port-other":"other",
            "protocol":"tcp",
            "detection-enable":1,
            "deny":1,
          },
          {
            "port-other":"other",
            "protocol":"udp",
            "detection-enable":1,
            "deny":1,
          }
        ]
      }
    }
  ]
}

    return ddos_dst_zone_payload 



import unittest,sys
sys.path.append('../')
from a10 import axapi_auth, axapi_action

a10_tps = "192.168.199.152"
username = "admin"
password = "a10"
hostname = "TH4435"

class Test_Auth(unittest.TestCase):

    def testAssertTrue(self):
        print("Testing axapi_auth")
        try:
            mitigator_base_url, signature = axapi_auth(a10_tps, username, password)
            print("base url: ", mitigator_base_url, "Signature: ", signature)
            axapi_action(mitigator_base_url+"/axapi/v3/logoff")
        
        except Exception as e:
            self.fail("Not authenticated")


class Test_API_Actions(unittest.TestCase):

    def testAssertTrue(self):
        try:
            print("Testing GET")
            mitigator_base_url, signature = axapi_auth(a10_tps, username, password)
            r = axapi_action(mitigator_base_url+"/axapi/v3/version/oper", method='GET', signature=signature)
            print(str(r))
            print("Testing POST")
            hostname_payload = {"hostname": {"value": hostname}}
            r = axapi_action(mitigator_base_url+"/axapi/v3/hostname", payload=hostname_payload, signature=signature)
            print(str(r))
            axapi_action(mitigator_base_url+"/axapi/v3/logoff")

        except Exception as e:
            self.fail("Failed")

if __name__ == "__main__":
    unittest.main()






ddos_dst_zone_path = '/axapi/v3/ddos/dst/zone/'

def ddos_dst_zone(zone_name, ip_addr):
    port_num = 53
    port_protocol = 'udp'
    ddos_dst_zone_payload = {
      "zone-list": [
        {
          "zone-name":zone_name,
          "ip": [
            {
              "ip-addr":ip_addr
            }
          ],
          "operational-mode":"monitor",
          "port": {
            "zone-service-list": [
              {
                "port-num":port_num,
                "protocol":port_protocol,
                "level-list": [
                  {
                    "level-num":"0",
                    "zone-escalation-score":1,
                    "indicator-list": [
                      {
                        "type":"pkt-rate",
                        "score":50,
                        "zone-threshold-num":1,
                      }
                    ],
                  },
                  {
                    "level-num":"1",
                  }
                ],
              }
            ],
          },
        }
      ]
    }  
    return ddos_dst_zone_payload 
